/*************************************************************************************************
 * Abstraction for database managers compabible with DBM
 *                                                      Copyright (C) 2000-2003 Mikio Hirabayashi
 * This file is part of QDBM, Quick DataBase Manager.
 * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation; either version 2 of the
 * License, or any later version.
 * QDBM is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with QDBM;
 * if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA.
 *************************************************************************************************/


package qdbm;


/**
 * Abstraction for database managers compatible with DBM.
 */
public interface ADBM {
  //----------------------------------------------------------------
  // methods
  //----------------------------------------------------------------
  /**
   * Close the database connection.
   * @throws DBMException if an error occures.
   */
  void close() throws DBMException;
  /**
   * Store a record.
   * @param key a byte array of a key.
   * @param val a byte array of a value.
   * @param replace whether the existing value is to be overwritten or not.
   * @throws DBMException if an error occures or replace is cancelled.
   */
  void store(byte[] key, byte[] val, boolean replace) throws DBMException;
  /**
   * Store a record.
   * @param key a serializable object of a key.
   * @param val a serializable object of a value.
   * @param replace whether the existing value is to be overwritten or not.
   * @throws DBMException if an error occures or replace is cancelled.
   */
  void storeobj(Object key, Object val, boolean replace) throws DBMException;
  /**
   * Store a record.
   * @param key the number of a key.
   * @param val a serializable object of a value.
   * @param replace whether the existing value is to be overwritten or not.
   * @throws DBMException if an error occures or replace is cancelled.
   */
  void storenum(long key, Object val, boolean replace) throws DBMException;
  /**
   * Delete a record.
   * @param key a byte array of a key.
   * @throws DBMException if an error occures or no record corresponds.
   */
  void delete(byte[] key) throws DBMException;
  /**
   * Delete a record.
   * @param key a serializable object of a key.
   * @throws DBMException if an error occures or no record corresponds.
   */
  void deleteobj(Object key) throws DBMException;
  /**
   * Delete a record.
   * @param key the number of a key.
   * @throws DBMException if an error occures or no record corresponds.
   */
  void deletenum(long key) throws DBMException;
  /**
   * Fetch a record.
   * @param key a byte array of a key.
   * @return a byte array of the value of the corresponding record.
   * @throws DBMException if an error occures or no record corresponds.
   */
  byte[] fetch(byte[] key) throws DBMException;
  /**
   * Fetch a record.
   * @param key a serializable object of a key.
   * @return an object of the value of the corresponding record.
   * @throws DBMException if an error occures or no record corresponds.
   */
  Object fetchobj(Object key) throws DBMException;
  /**
   * Fetch a record.
   * @param key the number of a key.
   * @return an object of the value of the corresponding record.
   * @throws DBMException if an error occures or no record corresponds.
   */
  Object fetchnum(long key) throws DBMException;
  /**
   * Get the first key.
   * @return a byte array of the key of the first record.
   * @throws DBMException if an error occures or no record corresponds.
   */
  byte[] firstkey() throws DBMException;
  /**
   * Get the first key.
   * @return an object of the key of the first record.
   * @throws DBMException if an error occures or no record corresponds.
   */
  Object firstkeyobj() throws DBMException;
  /**
   * Get the first key.
   * @return the number of the key of the first record.
   * @throws DBMException if an error occures or no record corresponds.
   */
  long firstkeynum() throws DBMException;
  /**
   * Get the next key.
   * @return a byte array of the key of the next record.
   * @throws DBMException if an error occures or no record corresponds.
   */
  byte[] nextkey() throws DBMException;
  /**
   * Get the next key.
   * @return an object of the key of the next record.
   * @throws DBMException if an error occures or no record corresponds.
   */
  Object nextkeyobj() throws DBMException;
  /**
   * Get the next key.
   * @return the number of the key of the next record.
   * @throws DBMException if an error occures or no record corresponds.
   */
  long nextkeynum() throws DBMException;
  /**
   * Check whether an fatal error occured or not.
   * @return true if the database has a fatal error, false if not.
   * @throws DBMException if an error occures.
   */
  boolean error() throws DBMException;
  /**
   * Clear the error condition on the database.
   * @throws DBMException if an error occures.
   */
  void clearerror() throws DBMException;
}



/* END OF FILE */
