/*************************************************************************************************
 * The B+-tree API of QDBM
 *                                                      Copyright (C) 2000-2003 Mikio Hirabayashi
 * This file is part of QDBM, Quick DataBase Manager.
 * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation; either version 2 of the
 * License, or any later version.
 * QDBM is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with QDBM;
 * if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA.
 *************************************************************************************************/


#ifndef _VILLA_H                         /* duplication check */
#define _VILLA_H


#include <depot.h>
#include <cabin.h>
#include <stdlib.h>



/*************************************************************************************************
 * API
 *************************************************************************************************/


typedef struct {                         /* type of structure for a record */
  CBDATUM *key;                          /* datum of the key */
  CBDATUM *first;                        /* datum of the first value */
  CBLIST *rest;                          /* list of the rest values */
} VLREC;

typedef struct {                         /* type of structure for index of a page */
  int pid;                               /* ID number of the referring page */
  CBDATUM *key;                          /* threshold key of the page */
} VLIDX;

typedef struct {                         /* type of structure for a leaf page */
  int id;                                /* ID number of the leaf */
  int dirty;                             /* whether to be written back or not */
  CBDATUM *key;                          /* threshold key of the page */
  CBLIST *recs;                          /* list of records */
  int prev;                              /* ID number of the previous leaf */
  int next;                              /* ID number of the next leaf */
} VLLEAF;

typedef struct {                         /* type of structure for a node page */
  int id;                                /* ID number of the node */
  int dirty;                             /* whether to be written back or not */
  int heir;                              /* ID of the child before the first index */
  CBLIST *recs;                          /* list of indexes */
} VLNODE;

/* type of the pointer to a comparing function.
   `aptr' specifies the pointer to the region of one key.
   `asiz' specifies the size of the region of one key.
   `bptr' specifies the pointer to the region of the other key.
   `bsiz' specifies the size of the region of the other key.
   The return value is positive if the former is big, negative if the latter is big, else, 0. */
typedef int(*VLCFUNC)(const char *aptr, int asiz, const char *bptr, int bsiz);

typedef struct {                         /* type of structure for a database handle */
  DEPOT *depot;                          /* internal database handle */
  VLCFUNC cmp;                           /* pointer to the comparing function */
  int wmode;                             /* whether writable or not */
  int root;                              /* ID number of the root page */
  int last;                              /* ID number of the last leaf */
  int lnum;                              /* number of leaves */
  int nnum;                              /* number of nodes */
  int rnum;                              /* number of records */
  CBMAP *leafc;                          /* cache for leaves */
  CBMAP *nodec;                          /* cache for nodes */
  int curleaf;                           /* ID number of the leaf where the cursor is */
  int curknum;                           /* index of the key where the cursor is */
  int curvnum;                           /* index of the value where the cursor is */
} VILLA;

enum {                                   /* enumeration for open modes */
  VL_OREADER = 1 << 0,                   /* open as a reader */
  VL_OWRITER = 1 << 1,                   /* open as a writer */
  VL_OCREAT = 1 << 2,                    /* a writer creating */
  VL_OTRUNC = 1 << 3                     /* a writer truncating */
};

enum {                                   /* enumeration for write modes */
  VL_DOVER,                              /* overwrite an existing value */
  VL_DKEEP,                              /* keep an existing value */
  VL_DDUP                                /* allow duplication of records */
};


/* Get a database handle. */
VILLA *vlopen(const char *name, int omode, VLCFUNC cmp);


/* Close a database handle. */
int vlclose(VILLA *villa);


/* Store a record. */
int vlput(VILLA *villa, const char *kbuf, int ksiz, const char *vbuf, int vsiz, int dmode);


/* Delete a record. */
int vlout(VILLA *villa, const char *kbuf, int ksiz);


/* Retrieve a record. */
char *vlget(VILLA *villa, const char *kbuf, int ksiz, int *sp);


/* Get the size of the value of a record. */
int vlvsiz(VILLA *villa, const char *kbuf, int ksiz);


/* Move the cursor to the first record. */
int vlcurfirst(VILLA *villa);


/* Move the cursor to the last record. */
int vlcurlast(VILLA *villa);


/* Move the cursor to the previous record. */
int vlcurprev(VILLA *villa);


/* Move the cursor to the next record. */
int vlcurnext(VILLA *villa);


/* Get the key of the record where the cursor is. */
char *vlcurkey(VILLA *villa, int *sp);


/* Get the value of the record where the cursor is. */
char *vlcurval(VILLA *villa, int *sp);


/* Synchronize updating contents with the file and the device. */
int vlsync(VILLA *villa);


/* Optimize a database. */
int vloptimize(VILLA *villa);


/* Get the name of a database. */
char *vlname(VILLA *villa);


/* Get the size of a database file. */
int vlfsiz(VILLA *villa);


/* Get the number of the leaves of B+-tree. */
int vllnum(VILLA *villa);


/* Get the number of the nodes of B+-tree. */
int vlnnum(VILLA *villa);


/* Get the number of the records stored in a database. */
int vlrnum(VILLA *villa);


/* Check whether a database handle is a writer or not. */
int vlwritable(VILLA *villa);


/* Check whether a database has a fatal error or not. */
int vlfatalerror(VILLA *villa);


/* Get the inode number of a database file. */
int vlinode(VILLA *villa);


/* Remove a database file. */
int vlremove(const char *name);



#endif                                   /* duplication check */


/* END OF FILE */
